pub trait Pattern<'a>:
HasTypeWitness<PatternInput<'a, Self>>
+ Copy
+ Sized { }Expand description
A string pattern.
Types that implement this trait can be used to search into a string.
This trait can only be implemented in the konst crate.
Dyn Compatibility
This trait is not dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".