Available on crate feature
iter only.Expand description
Const equivalent of iterators with a specific next function signature.
The docs for ConstIntoIter has more information on
const equivalents of IntoIterator and Iterator.
Re-exports
pub use crate::polymorphism::kinds::IsIntoIterKind;pub use crate::polymorphism::kinds::IsIteratorKind;pub use crate::polymorphism::kinds::IsStdKind;
Modules
- iterator_
dsl - Documentation on the iterator DSL that some
konst::itermacros support.
Macros
- collect_
const - Collects an iterator constant into an array
- eval
- Emulates iterator method chains, by expanding to equivalent code.
- for_
each - Iterates over all elements of an iterator,
const equivalent of
Iterator::for_each - into_
iter - Macro for converting
ConstIntoIterimplementors into const iterators.
Structs
- Into
Iter Wrapper - Wrapper for
ConstIntoIterimplementors, that defines different methods depending on the value ofK. - Repeat
- Const analog of
core::iter::Repeat, constructed byrepeat. - RepeatN
- Const analog of
core::iter::RepeatN, constructed byrepeat_n.
Traits
- Const
Into Iter - Const analog of the
IntoIteratortrait. - Const
Into Iter Kind - Trait for the types that are assignable to
ConstIntoIter::Kind - Step
- Trait for all the types that ranges can iterate over.
Functions
- coerce
- Coerces the argument to a type that has a
.const_into_iter()method - repeat
- Const analog of
core::iter::repeat, except that this requires the repeated value to implCopy(instead ofClone). - repeat_
n - Const analog of
core::iter::repeat_n, except that this requires the repeated value to implCopy(instead ofClone).